@@ -13,7 +13,7 @@
 
 # Source directory. Can be set here or as the first argument of the `sigal
 # build` command
-source = 'pictures'
+source = "pictures"
 
 # Destination directory. Can be set here or as the second argument of the
 # `sigal build` command (default: '_build')
@@ -22,7 +22,7 @@ source = 'pictures'
 # Theme :
 # - colorbox (default), galleria, photoswipe, or the path to a custom theme
 # directory
-theme = 'galleria'
+theme = "galleria"
 
 # Theme for galleria (https://galleriajs.github.io/themes/)
 # galleria_theme = 'classic'
@@ -119,7 +119,13 @@ thumb_size = (280, 210)
 # thumb_fit_centering = (0.5, 0.5)
 
 # Delay in seconds to avoid black thumbnails in videos with fade-in
-# thumb_video_delay = '0'
+# thumb_video_delay = 0
+# Max retries to generate a non-black thumbnail
+# thumb_video_black_retries = 0
+# For each retry, advance another N seconds on top of original delay
+# thumb_video_black_retry_offset = 1
+# A thumbnail with more than max_colors will not be considered "all black"
+# thumb_video_black_max_colors = 4
 
 # Keep original image (default: False)
 # keep_orig = True
