@@ -1,5 +1,5 @@
 # Copyright (c)      2013 - Christophe-Marie Duquesne
-# Copyright (c) 2013-2020 - Simon Conseil
+# Copyright (c) 2013-2023 - Simon Conseil
 # Copyright (c)      2021 - Keith Feldman
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -27,8 +27,9 @@ import shutil
 import subprocess
 from os.path import splitext
 
+from PIL import Image as PILImage
+
 from . import image, utils
-from .settings import Status
 from .utils import is_valid_html5_video
 
 
@@ -173,17 +174,43 @@ def generate_video(source, outname, settings):
 
 
 def generate_thumbnail(
-    source, outname, box, delay, fit=True, options=None, converter="ffmpeg"
+    source,
+    outname,
+    box,
+    delay,
+    fit=True,
+    options=None,
+    converter="ffmpeg",
+    black_retries=0,
+    black_offset=1,
+    black_max_colors=4,
 ):
     """Create a thumbnail image for the video source, based on ffmpeg."""
     logger = logging.getLogger(__name__)
     tmpfile = outname + ".tmp.jpg"
 
-    # dump an image of the video
-    cmd = [converter, "-i", source, "-an", "-r", "1"]
-    cmd += ["-ss", str(delay), "-vframes", "1", "-y", tmpfile]
-    logger.debug("Create thumbnail for video: %s", " ".join(cmd))
-    check_subprocess(cmd, source, outname)
+    currentTry = 0
+    iDelay = int(delay)
+    while currentTry <= abs(black_retries):
+        # dump an image of the video
+        cmd = [converter, "-i", source, "-an", "-r", "1"]
+        cmd += ["-ss", str(iDelay), "-vframes", "1", "-y", tmpfile]
+        logger.debug("Create thumbnail for video: %s", " ".join(cmd))
+        check_subprocess(cmd, source, outname)
+        if os.path.isfile(tmpfile) and black_retries > 0:
+            with PILImage.open(tmpfile) as img:
+                colors = img.getcolors(maxcolors=black_max_colors)
+            if colors is None:
+                # There were more colors than maxcolors in the image, it
+                # looks suitable for a valid thumbnail
+                break
+            else:
+                # Only found 'maxcolors' unique colors, looks like a solid
+                # color, try again with another seek delay
+                currentTry += 1
+                iDelay += abs(black_offset)
+        else:
+            break
 
     # Sometimes ffmpeg fails with returncode zero but without producing an
     # output file Thus, we need to check if an output file was created. If
@@ -206,7 +233,7 @@ def process_video(media):
     logger = logging.getLogger(__name__)
     settings = media.settings
 
-    try:
+    with utils.raise_if_debug() as status:
         if settings["use_orig"] and is_valid_html5_video(media.src_ext):
             utils.copy(media.src_path, media.dst_path, symlink=settings["orig_link"])
         else:
@@ -219,14 +246,8 @@ def process_video(media):
                 )
                 raise ValueError
             generate_video(media.src_path, media.dst_path, settings)
-    except Exception:
-        if logger.getEffectiveLevel() == logging.DEBUG:
-            raise
-        else:
-            return Status.FAILURE
 
-    if settings["make_thumbs"]:
-        try:
+        if settings["make_thumbs"]:
             generate_thumbnail(
                 media.dst_path,
                 media.thumb_path,
@@ -235,11 +256,9 @@ def process_video(media):
                 fit=settings["thumb_fit"],
                 options=settings["jpg_options"],
                 converter=settings["video_converter"],
+                black_retries=settings["thumb_video_black_retries"],
+                black_offset=settings["thumb_video_black_retry_offset"],
+                black_max_colors=settings["thumb_video_black_max_colors"],
             )
-        except Exception:
-            if logger.getEffectiveLevel() == logging.DEBUG:
-                raise
-            else:
-                return Status.FAILURE
 
-    return Status.SUCCESS
+    return status.value
